==> fv-code-mirror-editors-0.2.1.tgz: package/src/lib/common/ide.state.ts <==
import { BehaviorSubject, merge } from 'rxjs'
import { UpdateOrigin, SourceCode, SourceContent, SourcePath } from './models'
import { logFactory } from './log-factory.conf'
import { debounceTime, filter } from 'rxjs/operators'

const log = logFactory().getChildLogger('ide.state.ts')

export class IdeState {
    public readonly fsMap$ = new BehaviorSubject<Map<string, string>>(undefined)

    public readonly updates$: {
        [k: string]: BehaviorSubject<{
            path: SourcePath
            content: SourceContent
            updateOrigin: UpdateOrigin
        }>
    }

    constructor(params: {
        files: SourceCode[]
        defaultFileSystem: Promise<Map<string, string>>
    }) {
        Object.assign(this, params)

        this.fsMap$
            .pipe(filter((fsMap) => fsMap != undefined))
            .subscribe(() => log.info('IdeState => fsMap updated'))

        log.info('IdeState.constructor()')
        params.defaultFileSystem.then((defaultFsMap) => {
            const fsMap = new Map(defaultFsMap)
            params.files.forEach((file) => {
                fsMap.set(file.path, file.content)
            })
            log.info('IdeState => virtual files system initialized')
            this.fsMap$.next(fsMap)
        })
        this.updates$ = params.files.reduce((acc, e) => {
            return {
                ...acc,
                [e.path]: new BehaviorSubject({
                    path: e.path,
                    content: e.content,
                    updateOrigin: { uid: 'IdeState' },
                }),
            }
        }, {})
        merge(...Object.values(this.updates$))
            .pipe(debounceTime(250))
            .subscribe(({ path, content }) => {
                const fsMap = this.fsMap$.getValue()
                fsMap && fsMap.set(path, content)
                fsMap && this.fsMap$.next(fsMap)
            })
    }

    update({
        path,
        content,
        updateOrigin,
    }: {
        path: SourcePath
        content: SourceContent
        updateOrigin: UpdateOrigin
    }) {
        this.updates$[path].next({
            path,
            content,
            updateOrigin: updateOrigin,
        })
    }
}

==> fv-code-mirror-editors-0.3.1.tgz: package/src/lib/common/ide.state.ts <==
import {BehaviorSubject} from 'rxjs'
import { UpdateOrigin, SourceCode, SourceContent, SourcePath } from './models'
import { logFactory } from './log-factory.conf'
import {debounceTime, filter} from 'rxjs/operators'

const log = logFactory().getChildLogger('ide.state.ts')

export class IdeState {
    public readonly fsMap$ = new BehaviorSubject<Map<string, string>>(undefined)

    public readonly updates$: {
        [k: string]: BehaviorSubject<{
            path: SourcePath
            content: SourceContent
            updateOrigin: UpdateOrigin
        }>
    }

    constructor(params: {
        files: SourceCode[]
        defaultFileSystem: Promise<Map<string, string>>
    }) {
        Object.assign(this, params)

        this.fsMap$
            .pipe(
                filter((fsMap) => fsMap != undefined),
                debounceTime(500)
            )
            .subscribe(() => log.info('IdeState => fsMap updated'))

        log.info('IdeState.constructor()')
        params.defaultFileSystem.then((defaultFsMap) => {
            const fsMap = new Map(defaultFsMap)
            params.files.forEach((file) => {
                fsMap.set(file.path, file.content)
            })
            log.info('IdeState => virtual files system initialized')
            this.fsMap$.next(fsMap)
        })
        this.updates$ = params.files.reduce((acc, e) => {
            return {
                ...acc,
                [e.path]: new BehaviorSubject({
                    path: e.path,
                    content: e.content,
                    updateOrigin: { uid: 'IdeState' },
                }),
            }
        }, {})
    }

    update({
        path,
        content,
        updateOrigin,
    }: {
        path: SourcePath
        content: SourceContent
        updateOrigin: UpdateOrigin
    }) {

        const fsMap = this.fsMap$.value
        fsMap.set(path, content)
        this.fsMap$.next(fsMap)
        this.updates$[path].next({
            path,
            content,
            updateOrigin: updateOrigin,
        })
    }

    addFile(source: SourceCode){
        const fsMap = this.fsMap$.value
        fsMap.set(source.path, source.content)
        this.fsMap$.next(fsMap)
        this.updates$[source.path] = new BehaviorSubject({...source, updateOrigin:{ uid: 'IdeState' }})
    }

    removeFile(path: string){
        const fsMap = this.fsMap$.value
        fsMap.delete(path)
        this.fsMap$.next(fsMap)
        delete this.updates$[path]
    }

    moveFile(oldPath: string, newPath: string){
        const fsMap = this.fsMap$.value
        const content =  fsMap.get(oldPath)
        fsMap.set(newPath, content)
        fsMap.delete(oldPath)
        this.fsMap$.next(fsMap)
        delete this.updates$[oldPath]
        this.updates$[newPath] = new BehaviorSubject({path: newPath, content, updateOrigin:{ uid: 'IdeState' }})
    }
}
